/* global LayoutManager, utils, LibraryManager, Logger */

/**
 * Display a HTML widget
 * @syntax new LayoutManager.html(media, container, callback, layoutid, scp)
 * @param {type} media
 * @param {type} container
 * @param {type} callback
 * @param {type} layoutid
 * @param {type} scp
 * @version 1.8.1
 * @since 1.8.1
 */
(function() {
	"use strict";

	window.LayoutManager.html = function (media, container, callback, layoutid, scheduleid, scp) {
		if (media.media.audio !== undefined) {
			var audio = {
					uri : "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
					loop : media.media.audio[0].uri[0].jAttr.loop,
					volume : media.media.audio[0].uri[0].jAttr.volume
			};
			this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
			this.audiooverlay.preload(function(){
				parent.audiooverlay.play();
			}, null);
		}

		this.statsStartTime = utils.getISODate();
		this.duration = parseInt(media.duration);
		var duration = this.duration;
		this.uri = "file:///opt/usr/home/owner/content/Downloads/" + media.id + ".html";
		var fragment = document.createDocumentFragment();
		this.elem = document.createElement("iframe");
		this.elem.src = this.uri;
		this.elem.style.position = "absolute";
		this.elem.style.left = 0;
		this.elem.style.top = 0;
		this.elem.style.width = "100%";
		this.elem.style.height = "100%";
		this.elem.style.overflow = "hidden";
		this.elem.style.border = "none";
		var parent = this;
		this.timeoutElement = null;
		this.removeMedia = function () { container.removeChild(this.elem); };
		var hastransIn = false;
		var hastransOut = false;
		var transInDuration = 1000;
		var transOutDuration = 1000;
		var transInType = null;
		var transOutType = null;
		var transInDirection = "S";
		var transOutDirection = "S";

		// Fix for XLG-57
		var checkUpdateInterval = function(){
			var invalidateAfter = 300;

			var f=LibraryManager.getFileByName(media.id+".html");
			if (f !== null)
			{
				try {
					if (media.media.options[0].hasOwnProperty("updateInterval"))
					{
						invalidateAfter = parseInt(media.media.options[0].updateInterval[0].jValue)*60;
					}
					if (utils.getTimeStamp()>= (parseInt(f.lastChecked)+invalidateAfter))
					{
						f.manuallyInvalidated = true;
						f.download(function(){
							//console.log("Refreshing resource " + f.filename);
							f.lastChecked = utils.getTimeStamp();
						},function(cbErr){
							console.warn("Something wend wrong for resource "+ f.filename, cbErr);
						});
					}else{
						//console.log("Resource " + f.filename + " will update after "+ ((parseInt(f.lastChecked)+invalidateAfter) - utils.getTimeStamp()) +" seconds.");
					}

				} catch (e) {
	                console.warn('Error for file '+f.filename,e);
				}
			}
		};

		if (media.media.options[0].hasOwnProperty("transIn")){ hastransIn = true; transInType = media.media.options[0].transIn[0].jValue; transInDirection = media.media.options[0].transInDirection[0].jValue; }
		if (media.media.options[0].hasOwnProperty("transOut")){ hastransOut = true; transOutType = media.media.options[0].transOut[0].jValue; transOutDirection = media.media.options[0].transOutDirection[0].jValue; }
		if (media.media.options[0].hasOwnProperty("transInDuration")){ transInDuration = media.media.options[0].transInDuration[0].jValue; }
		if (media.media.options[0].hasOwnProperty("transOutDuration")){ transOutDuration = media.media.options[0].transOutDuration[0].jValue; }
		this.timeoutMedia = function () {
			checkUpdateInterval();
			//var tag = "r: " + container.getAttribute("id") + " m: " + media.id + " html";
			parent.timeoutElement = setTimeout(function(){
				if (hastransOut) {
					LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function() { 
						if (parent.audiooverlay){
							parent.audiooverlay.stop();
						}
						if (scheduleid){
	                        Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
	                    }
						callback(); 
					});
				} else {
					if (parent.audiooverlay){
						parent.audiooverlay.stop();
					}
					if (scheduleid){
                        Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
                    }
					callback();
				}
			}, duration * 1000, layoutid, scp);
		};
		var timeoutMediaInvoke = this.timeoutMedia;
		this.displayThisMedia = function () {
			if (hastransIn) { duration -= parseInt(transInDuration)/1000; utils.hide(this.elem);}
			if (hastransOut) { duration -= parseInt(transOutDuration)/1000; }
			if (duration < 0) { duration = 0; }
			container.innerHTML = null;
			fragment.appendChild(this.elem);
			container.appendChild(fragment);
			if (hastransIn) {
				LayoutManager.transin(this.elem, container, transInType, transInDuration, transInDirection, function() {timeoutMediaInvoke(parent);});
			} else {
				timeoutMediaInvoke(parent);
			}
		};
		this.stopMediaTimeout = function () {
			if (parent.audiooverlay){
				parent.audiooverlay.stop();
			}
//			if (scheduleid){
//                Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
//            }
			clearTimeout(parent.timeoutElement);
		};
		utils.httpGetAsync(this.uri, function(contents){
			if (contents.indexOf("<!-- DURATION=") !== -1) {
				var parts = contents.split("<!-- DURATION=");
				var htmlduration = parseInt(parts[parts.length - 1]);
				if (htmlduration > 0) { duration = htmlduration; }
			}
			parent.displayThisMedia(parent);        
		}, function(errorCb){
			console.error(errorCb);
		});
	};
}());