/* global DeviceManager, utils, ko */
(function(){
"use strict";

	/**
	 * @classdesc Status page view model.
	 * @type statusViewModel
	 * @version 1.8.1
	 * @since 1.8.1
	 */
	window.statusViewModel = function _statusViewModel(){
		var self = this;
		
		var v_clearLogTimer = null;
		
		self.filterById = function(id){
			if (typeof(id)==='undefined'){
				return this.allLayoutsSet();
			}else{
				return ko.utils.arrayFilter(this.allLayoutsSet(),function(item){
					return item.name === id;
				});
			}
		}

		self.filterByScheduled = function(isScheduled){
			if (typeof(isScheduled)==='undefined'){
				return this.allLayoutsSet();
			}else{
				return ko.utils.arrayFilter(this.allLayoutsSet(),function(item){
					return item.isScheduled() === isScheduled;
				});
			}
		}
		self.filterByValid = function(isValid){
			if (typeof(isValid)==='undefined'){
				return this.allLayoutsSet();
			}else{
				return ko.utils.arrayFilter(this.allLayoutsSet(),function(item){
					return item.isValid() === isValid;
				});
			}
		}
		self.filterByOverlay = function(isOverlay){
			if (typeof(isOverlay)==='undefined'){
				return this.allLayoutsSet();
			}else{
				return ko.utils.arrayFilter(this.allLayoutsSet(),function(item){
					return item.isOverlay() === isOverlay;
				});
			}
		}

		/**
		 * Create a layout status object.
		 * 
		 * @param {string} name The name (id) of the layout.
		 * @param {boolean} isDefault Flag for default layout. 
		 * @param {boolean} isValid Flag for valid layout.
		 * @param {boolean} isScheduled Flag if layout is sceduled.
		 * @param {boolean} isOverlay Flag if overlay.
		 */
		self.createLayoutInfo = function(name,isDefault,isValid,isScheduled,isOverlay){
			var valid;
			if(typeof(isValid) === "boolean"){
				valid=isValid
			}else if (typeof(isValid)==='undefined'){
				valid = typeof(isValid)=== 'undefined'?false:isValid;
			}else{
				valid=(isValid===1)?true:false
			}
			isDefault = typeof(isDefault)=== 'undefined'?false:isDefault;
			isScheduled = typeof(isScheduled)=== 'undefined'?false:isScheduled;
			isOverlay = typeof(isOverlay)=== 'undefined'?false:isScheduled;
			
			var layoutInfo = {
				name:name,
				isDefault:ko.observable(isDefault), 
			 	isValid:ko.observable(valid),
			 	isScheduled:ko.observable(isScheduled),
			 	isOverlay:ko.observable(isOverlay) 
			}
			
			layoutInfo.toString = ko.computed(function() {
				var str = layoutInfo.name;
				if (!layoutInfo.isScheduled()){
					str+="*";
				}
				if (layoutInfo.isDefault()){
					str+=" (D)";
				}
				if (layoutInfo.isOverlay()){
					str+=" (O)";
				}
				return str;
			});
			
			return layoutInfo;
		};

		self.appVersion = ko.observable(DeviceManager.Device.getPlatformInfo().clientVersion);
		self.appVerCode = ko.observable(DeviceManager.Device.getPlatformInfo().clientCode);
		self.cmsAddr = ko.observable(DeviceManager.Config.getURL());
	    self.isDevVersion = ko.pureComputed(function() {
	      return AppVersionInformation.isDevVersion;
	    });
	    
	    self.patchVersion = ko.pureComputed(function() {
	      return AppVersionInformation.patchVersion;
	    });
		self.webOSVersion = DeviceManager.Device.webOSVersion;

		self.deviceModel = ko.observable();
	    self.firmwareVersion = ko.observable();
	    
		self.xmrAddr = ko.observable();
		self.lastXMRTimestamp = ko.observable();
		self.playerDateTime = ko.observable();
//		self.xmrAddrDispl = ko.pureComputed(function(){
//		return "("+self.xmrAddr()+")";
//		});

		self.memoryTotal = ko.observable();
		self.memoryFree = ko.observable();
		self.memoryUsed = ko.observable();

		self.spaceTotal = ko.observable();
		self.spaceFree = ko.observable();
		self.spaceUsed = ko.observable();

		self.spaceAvail = ko.pureComputed(function(){
			//jshint -W016
			return ( ( ( self.spaceFree()  / self.spaceTotal() ) * 100) |0 ) + "%";
			//jshint +W016
		});

		self.displayName = ko.observable(DeviceManager.Config.getDisplayName());

		self.mainLicenseRemain = ko.observable();
		self.trialLicenseRemain = ko.observable();

		self.licenseValue = ko.computed(function(){
			if (self.mainLicenseRemain()>utils.getTimeStamp()){
				return "Licensed.";
			}else if (self.trialLicenseRemain()>utils.getTimeStamp()){
				var remain = new Date((self.trialLicenseRemain()-utils.getTimeStamp())*1000);
				return "Trial (Remain " + remain.getDate() + " days).";
			}else{
				return "No license.";
			}
		});

		self.cmsRegistered = ko.observable(false);
		self.currentLayout = ko.observable();
		self.screenSize = ko.observable();
		self.screenShotInterval = ko.observable("0");

		self.cmsRegInfo = ko.observable("Not registered.");
		self.cmsRegInfoTime = ko.observable();

		self.scheduleStatus = ko.observable();
		//self.reqFilesStatus = ko.observable();
		self.downloadedFiles = ko.observable(0);
		self.totalFiles = ko.observable(0);
		self.remainingFiles = ko.observable(0); // Queued Number Connections
		self.failedXMDSConnections = ko.observable(0);

		self.nextLayout = ko.observable();
//	    self.scheduledLayouts_default = ko.observable().extend({ rateLimit: 100 });;
//	    self.scheduledLayouts_schedule = ko.observableArray().extend({ rateLimit: 100 });;
//	    self.scheduledLayouts_overlay = ko.observableArray().extend({ rateLimit: 100 });;
	    
	    /** Valid Layouts */
	    self.validLayouts_default = ko.observable().extend({ rateLimit: 100 });; 
	    self.validLayouts_schedule = ko.observableArray().extend({ rateLimit: 100 });
	    self.validLayouts_overlay = ko.observableArray().extend({ rateLimit: 100 });
	    
	    /** Invalid Layouts */
	    self.invalidLayouts_default = ko.observable().extend({ rateLimit: 100 });; 
	    self.invalidLayouts_schedule = ko.observableArray().extend({ rateLimit: 100 });
	    self.invalidLayouts_overlay = ko.observableArray().extend({ rateLimit: 100 });

	    self.allLayoutsSet = ko.observableArray().extend({ rateLimit: 100 });
	    
	    
	      //self.allLayouts = ko.observable();/
	    /* All Layouts */
	    self.allLayouts = ko.computed(function(){
	    	return self.allLayoutsSet();
//	    	
//	        var a = [];
////	        if (self.validLayouts_default() !== undefined && self.validLayouts_default() !== ""){
////	          a.push(self.validLayouts_default());
////	        }
//	        ko.utils.arrayForEach(self.allLayoutsSet(), function (item) {
//	          var str = item.name;
//	          if (!item.isScheduled()){
//	        	  str+="*";
//	          }
//	          if (item.isDefault()){
//	        	  str+=" (D)";
//	          }
//        	  a.push(str);
//	        });

//	        if (self.invalidLayouts_default() !== undefined && self.invalidLayouts_default() !== ""){
//	          a.push(self.invalidLayouts_default());
//	        }
//	        ko.utils.arrayForEach(self.validLayouts_schedule(), function (item) {
//	          a.push(item);
//	        });
//	        ko.utils.arrayForEach(self.invalidLayouts_schedule(), function (item) {
//	          a.push(item);
//	        });
//	        ko.utils.arrayForEach(self.validLayouts_overlay(), function (item) {
//	          a.push(item);
//	        });
//	        ko.utils.arrayForEach(self.invalidLayouts_overlay(), function (item) {
//	          a.push(item);
//	        });

//	        return a;
	    });
	    
	    /* Scheduled */
	    //self.scheduledLayouts = ko.observable();
	    self.scheduledLayouts = ko.computed(function(){
	    	return self.filterByScheduled(true);
//	    	var a = [];
//	        if (self.validLayouts_default() !== undefined && self.validLayouts_default() !== ""){
//	          a.push(self.validLayouts_default());
//	        }
//	        ko.utils.arrayForEach(self.filterByScheduled(true), function (item) {
//	          var str = item.name;
//	          if (item.isDefault()){
//	        	  str+=" (D)";
//	          }
//        	  a.push(str);
//	        });
//	        var a = [];
//	        if (self.scheduledLayouts_default() !== undefined && self.scheduledLayouts_default() !== ""){
//	          a.push(self.scheduledLayouts_default());
//	        }
//	        ko.utils.arrayForEach(self.scheduledLayouts_schedule(), function (item) {
//	          a.push(item);
//	        });
//	        ko.utils.arrayForEach(self.scheduledLayouts_overlay(), function (item) {
//	          a.push(item);
//	        });
//	        return a;
	    }).extend({ rateLimit: 100 });

	    /* Valid Layouts */
	    self.validLayouts = ko.computed(function(){
	    	var a = [];
	    	ko.utils.arrayForEach(self.filterByValid(true), function (item) {
	    		a.push(item.name);
	    	});
	    	return ko.utils.arrayGetDistinctValues(a);
	    	//return self.filterByValid(true);
	    });
	    
	    /* Invalid Layouts */
	    self.invalidLayouts = ko.computed(function(){
	    	var a = [];
	    	ko.utils.arrayForEach(self.filterByValid(false), function (item) {
	    		a.push(item.name);
	    	});
	    	return ko.utils.arrayGetDistinctValues(a);
	    	//return self.filterByValid(false);
//	        var a = [];
//	        if (self.invalidLayouts_default() !== undefined && self.invalidLayouts_default() !== ""){
//	          a.push(self.invalidLayouts_default());
//	        }
//	        ko.utils.arrayForEach(self.invalidLayouts_schedule(), function (item) {
//	          a.push(item);
//	        });
//	        ko.utils.arrayForEach(self.invalidLayouts_overlay(), function (item) {
//	          a.push(item);
//	        });
//	        return a;
	    });

		self.blacklist = ko.observableArray();

		/**
		 * Contains the last 5 log messages. Don't use to push logs, use {@see addLogMessage} for that.
		 */
		self.logMessages = ko.observableArray();

		self.updateWindowStart = ko.observable();
		self.updateWindowEnd = ko.observable();
		self.versionInformationCMS = ko.observable();

		self.addLogMessage = function(logEntry){
			if (self.logMessages().length>=5) { self.logMessages.shift(); }
			self.logMessages.push(logEntry);
		};


		self.timelyClearLogs = function(){
			if (v_clearLogTimer !== null){
				clearTimeout(v_clearLogTimer);
			}

			/* Automatic clear  */
			setTimeout(function(){
				self.logMessages.removeAll();
			},5*60*1000);
		};
		//<div>Image Cache Hit Rate: <span>0.25</span></div>
		//<div>Use Surface View for Video Rendering?: <span>true</span></div>

	};
}());